import java.io.*; // Para que encuentre File
import java.awt.image.*; // Para que encuentre BufferedImage
import javax.imageio.*; // para que encuentre ImageIO

// Clase para imagen en color
class ImagenRGB
{
	public int[][][] pix;
	int lx;
	int ly;

	public ImagenRGB(int lx, int ly)
	{
		this.pix=new int[lx][ly][3];
		this.lx=lx;
		this.ly=ly;
	}
	public ImagenRGB(String nombreArchivo) throws IOException
	{
		File infile = new File(nombreArchivo);
		BufferedImage im = ImageIO.read(infile);
		int r, g, b, rgb;
		int i, j;

		this.lx=im.getWidth();
		this.ly=im.getHeight();
		this.pix=new int[this.lx][this.ly][3];
		for (i=0; i<this.lx; i++)
		{
			for (j=0; j<this.ly; j++)
			{
				rgb=im.getRGB(i,j); // color RGB mezclado: R:bits 16-23, B:bits 8-15, B:bits 0-7

				// Ahora, separar las componentes
          		r=(rgb/256/256)&255;//bits 16-23. El &255 deja los ultimos 8 bits.
          		g=(rgb/256)&255; //bits 8-15
          		b=rgb&255;      //bits 0-7

          		this.pix[i][j][0]=r;
          		this.pix[i][j][1]=g;
          		this.pix[i][j][2]=b;
			}
		}
	}
	public void graba(String nombreArchivo, String tipo) throws IOException
	{
		BufferedImage im2=new BufferedImage(this.lx, this.ly, BufferedImage.TYPE_INT_RGB); // nueva imagen RGB

		int i, j;
		int r, g, b;
		int rgb;
		for (i=0; i<this.lx; i++)
		{
			for (j=0; j<this.ly; j++)
			{
				r=this.pix[i][j][0]; // color rojo se copia
          		g=this.pix[i][j][1]; // color verde se copia
          		b=this.pix[i][j][2]; // color azul se copia

          		rgb=r*256*256+g*256+b; // Volver a mezclar las componentes
          		im2.setRGB(i,j,rgb);
			}
		}
		File outfile = new File(nombreArchivo);
		ImageIO.write(im2, tipo, outfile);
	}
}

// Clase para imagen en blanco y negro
class ImagenBN
{
	public int[][] pix;
	public int lx;
	public int ly;

	public static int calcGris(int r, int g, int b) {return (int)( r*0.299 + g*0.511 + b*0.114 );}
	public ImagenBN(int lx, int ly)
	{
		this.pix=new int[lx][ly];
		this.lx=lx;
		this.ly=ly;
	}
	public ImagenBN(String nombreArchivo) throws IOException
	{
		File infile = new File(nombreArchivo);
		BufferedImage im = ImageIO.read(infile);
		int r, g, b, rgb;
		int i, j;


		this.lx=im.getWidth();
		this.ly=im.getHeight();
		this.pix=new int[this.lx][this.ly];
		for (i=0; i<this.lx; i++)
		{
			for (j=0; j<this.ly; j++)
			{
				rgb=im.getRGB(i,j); // color RGB mezclado: R:bits 16-23, B:bits 8-15, B:bits 0-7

				// Ahora, separar las componentes
          		r=(rgb/256/256)&255;//bits 16-23. El &255 deja los ultimos 8 bits.
          		g=(rgb/256)&255; //bits 8-15
          		b=rgb&255;      //bits 0-7

          		this.pix[i][j]=calcGris(r,g,b); // trasformar a tonos de gris
			}
		}
	}
	public void graba(String nombreArchivo, String tipo) throws IOException
	{
		BufferedImage im2=new BufferedImage(this.lx, this.ly, BufferedImage.TYPE_INT_RGB); // nueva imagen RGB

		int i, j;
		int r, g, b;
		int rgb;
		for (i=0; i<this.lx; i++)
		{
			for (j=0; j<this.ly; j++)
			{
				r=this.pix[i][j]; // color rojo se copia
          		g=this.pix[i][j]; // color verde se copia
          		b=this.pix[i][j]; // color azul se copia

          		rgb=r*256*256+g*256+b; // Volver a mezclar las componentes
          		im2.setRGB(i,j,rgb);
			}
		}
		File outfile = new File(nombreArchivo);
		ImageIO.write(im2, tipo, outfile);
	}
}

// Ejemplo que usa las clases
public class EjemploJava2
{
	////////////////////////////
	// Ahora el programa de ejemplo. Todo lo anterior es para poder definir como trabajar con imagenes.
	public static void main(String[] args) throws IOException
	{
		ImagenRGB im, im2;
		int i, j;

		im=new ImagenRGB("mono.jpg");
		im2=new ImagenRGB(im.lx, im.ly);

		// Ejemplo: Copiar slo canalazul
		for (i=0; i<im.lx; i++)
		{
			for (j=0; j<im.ly; j++)
			{
				im2.pix[i][j][0]=0; // color rojo se deja en "negro"
				im2.pix[i][j][1]=0; // color verde se deja en "negro"
				im2.pix[i][j][2]=im.pix[i][j][2]; // color azul se copia
			}
		}

		im2.graba("mono2.jpg","jpg");

		ImagenBN im3=new ImagenBN(im.lx, im.ly);

		// Ejemplo: pasar una imagen a blanco y negro
		for (i=0; i<im.lx; i++)
		{
			for (j=0; j<im.ly; j++)
			{
				im3.pix[i][j]=ImagenBN.calcGris(im.pix[i][j][0],im.pix[i][j][1],im.pix[i][j][2]);
			}
		}

		ImagenBN im4=new ImagenBN(im.lx, im.ly);

		// Ejemplo: Negativo de una imagen en blanco y negro
		for (i=0; i<im.lx; i++)
		{
			for (j=0; j<im.ly; j++)
			{
				im4.pix[i][j]=255 - im3.pix[i][j];
			}
		}

		im4.graba("mono3.jpg","jpg");
		return;
	}
}



